@@ -1,5 +1,6 @@
 package com.thoughtworks.qdox.model;
 
+import com.thoughtworks.qdox.JavaClassContext;
 import com.thoughtworks.qdox.model.util.OrderedMap;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -18,6 +19,7 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
 
     private static Type OBJECT = new Type("java.lang.Object");
     private static Type ENUM = new Type("java.lang.Enum");
+    private static Type ANNOTATION = new Type("java.lang.annotation.Annotation");
 
     private List methods = new LinkedList();
     private JavaMethod[] methodsArray;
@@ -27,12 +29,14 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
     private JavaClass[] classesArray;
     private boolean interfce;
     private boolean isEnum;
+    private boolean isAnnotation;
 
     // Don't access this directly. Use asType() to get my Type
     private Type type;
     private Type superClass;
     private Type[] implementz = new Type[0];
-    private JavaClassCache javaClassCache;
+    private JavaClassContext context;
+	private JavaPackage javaPackage;
 
     public JavaClass() {
     }
@@ -41,13 +45,13 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         setName(name);
     }
 
-    public void setJavaClassCache(JavaClassCache javaClassCache) {
-        this.javaClassCache = javaClassCache;
+    public void setJavaClassContext(JavaClassContext context) {
+		this.context = context;
 
         // reassign OBJECT. This will make it have a "source" too,
         // causing Type.getJavaClass() to return a JavaClass, instead
         // of null.
-        OBJECT = javaClassCache.getClassByName("java.lang.Object").asType();
+        OBJECT = context.getClassByName("java.lang.Object").asType();
     }
 
     /**
@@ -69,7 +73,7 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
 
         if (isEnum) {
             return ENUM;
-        } else if (!interfce && (superClass == null) && !iAmJavaLangObject) {
+        } else if (!interfce && !isAnnotation && (superClass == null) && !iAmJavaLangObject) {
             return OBJECT;
         }
 
@@ -109,7 +113,7 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         writeAccessibilityModifier(result);
         writeNonAccessibilityModifiers(result);
 
-        result.write(isEnum ? "enum " : interfce ? "interface " : "class ");
+        result.write(isEnum ? "enum " : interfce ? "interface " : isAnnotation ? "@interface " : "class ");
         result.write(name);
 
         // subclass
@@ -173,6 +177,10 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         this.isEnum = isEnum;
     }
 
+    public void setAnnotation(boolean isAnnotation) {
+        this.isAnnotation = isAnnotation;
+    }
+
     public void addMethod(JavaMethod meth) {
         meth.setParent(this);
         methods.add(meth);
@@ -193,18 +201,22 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         fields.add(javaField);
         fieldsArray = null;
     }
+    
+    public void setJavaPackage(JavaPackage javaPackage) {
+    	this.javaPackage = javaPackage;
+    }
 
     public JavaSource getParentSource() {
         JavaClassParent parent = getParent();
         return ((parent == null) ? null : parent.getParentSource());
     }
 
-    public String getPackage() {
-        return getParentSource().getPackage();
+    public JavaPackage getPackage() {
+        return (getParentSource() != null ? getParentSource().getPackage() : javaPackage);
     }
 
     public String getFullyQualifiedName() {
-        return getParent().getClassNamePrefix() + getName();
+        return (getParent() != null ? (getParent().getClassNamePrefix()) : javaPackage != null ? (javaPackage.getName()+".") : "") + getName();
     }
 
     /**
@@ -226,8 +238,8 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         return getParent().resolveType(typeName);
     }
 
-    public ClassLibrary getClassLibrary() {
-        return getParent().getClassLibrary();
+    public JavaClassContext getJavaClassContext() {
+        return getParent().getJavaClassContext();
     }
 
     public String getClassNamePrefix() {
@@ -418,13 +430,20 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
 
     public JavaClass getNestedClassByName(String name) {
         JavaClass[] classes = getNestedClasses();
-
+        
+        int separatorIndex = name.indexOf('.');
+        String directInnerClassName = (separatorIndex > 0 ? name.substring(0, separatorIndex) : name); 
         for (int i = 0; i < classes.length; i++) {
-            if (classes[i].getName().equals(name)) {
-                return classes[i];
+        	JavaClass jClass = classes[i];
+            if (jClass.getName().equals(directInnerClassName)) {
+            	if(separatorIndex > 0) {
+                    return jClass.getNestedClassByName(name.substring(separatorIndex+1));
+            	}
+            	else {
+                    return jClass;
+            	}
             }
         }
-
         return null;
     }
 
@@ -539,7 +558,7 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
      */
     public JavaClass[] getDerivedClasses() {
         List result = new ArrayList();
-        JavaClass[] classes = javaClassCache.getClasses();
+        JavaClass[] classes = context.getClasses();
 
         for (int i = 0; i < classes.length; i++) {
             JavaClass clazz = classes[i];
@@ -599,4 +618,20 @@ public class JavaClass extends AbstractInheritableJavaEntity implements JavaClas
         return getFullyQualifiedName().compareTo(((JavaClass) o).getFullyQualifiedName());
     }
 
+    /**
+     * @see http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html#toString()
+     */
+    public String toString() {
+    	StringBuffer sb = new StringBuffer();
+    	if(asType().isPrimitive() || (Type.VOID.equals(asType()))) {
+    		sb.append(asType().getValue());
+    	}
+    	else {
+        	sb.append(isInterface() ? "interface" : "class");
+        	sb.append(" ");
+        	sb.append(getFullyQualifiedName());
+    	}
+    	return sb.toString();
+    }
+
 }
