@@ -55,5 +55,23 @@ public class JavaParameter implements Serializable {
     public boolean isVarArgs() {
         return varArgs;
     }
+    
+    public String toString() {
+    	return getResolvedValue() + " "+ name;
+    }
+    
+    /**
+     * 
+     * @return the resolved value if the method has typeParameters, otherwise type's value
+     * @since 1.10
+     */
+    public String getResolvedValue() {
+		return type.getResolvedValue(getParentMethod().getTypeParameters());
+    }
+
+	public String getResolvedGenericValue() {
+		return type.getResolvedGenericValue(getParentMethod().getTypeParameters());
+	}
+	
 
 }
