@@ -3,7 +3,7 @@ package com.thoughtworks.qdox;
 import com.thoughtworks.qdox.model.*;
 import com.thoughtworks.qdox.model.util.SerializationUtils;
 import com.thoughtworks.qdox.parser.ParseException;
-import junit.framework.TestCase;
+import com.thoughtworks.qdox.testdata.PropertyClass;
 
 import java.io.*;
 import java.util.Arrays;
@@ -11,25 +11,29 @@ import java.util.List;
 import java.util.Set;
 import java.util.HashSet;
 
+import org.jmock.Mock;
+import org.jmock.MockObjectTestCase;
+
 /**
  * @author <a href="mailto:joew@thoughtworks.com">Joe Walnes</a>
  * @author Aslak Helles&oslash;y
  */
-public class JavaDocBuilderTest extends TestCase {
+public class JavaDocBuilderTest extends MockObjectTestCase {
 
     private JavaDocBuilder builder;
 
-    public JavaDocBuilderTest(String name) {
-        super(name);
-    }
-
     protected void setUp() throws Exception {
         super.setUp();
         builder = new JavaDocBuilder();
-        createFile("tmp/sourcetest/com/blah/Thing.java", "com.blah", "Thing");
-        createFile("tmp/sourcetest/com/blah/Another.java", "com.blah", "Another");
-        createFile("tmp/sourcetest/com/blah/subpackage/Cheese.java", "com.blah.subpackage", "Cheese");
-        createFile("tmp/sourcetest/com/blah/Ignore.notjava", "com.blah", "Ignore");
+        createFile("target/test-source/com/blah/Thing.java", "com.blah", "Thing");
+        createFile("target/test-source/com/blah/Another.java", "com.blah", "Another");
+        createFile("target/test-source/com/blah/subpackage/Cheese.java", "com.blah.subpackage", "Cheese");
+        createFile("target/test-source/com/blah/Ignore.notjava", "com.blah", "Ignore");
+    }
+
+    protected void tearDown() throws Exception {
+        super.tearDown();
+        deleteDir("target/test-source");
     }
 
     public void testParsingMultipleJavaFiles() {
@@ -45,7 +49,6 @@ public class JavaDocBuilderTest extends TestCase {
         JavaClass testClass = sources[1].getClasses()[0];
         assertEquals("TestClass", testClass.getName());
 
-
         JavaClass testClassListByName = builder.getClassByName("com.thoughtworks.qdox.TestClassList");
         assertEquals("TestClassList", testClassListByName.getName());
 
@@ -104,6 +107,30 @@ public class JavaDocBuilderTest extends TestCase {
         assertEquals(2, classes.length);
     }
 
+    public void testGetPackagesShowsOnePackageAndTwoClasses() {
+        builder.addSourceTree(new File("target/test-source"));
+        JavaPackage[] packages = builder.getPackages();
+        assertEquals(2, packages.length);
+        JavaPackage comBlahSubpackage = packages[0];
+        assertEquals("com.blah.subpackage", comBlahSubpackage.getName());
+        JavaPackage comBlah = packages[1];
+        assertEquals("com.blah", comBlah.getName());
+        JavaClass[] classes = comBlahSubpackage.getClasses();
+        assertEquals(1, classes.length);
+        assertEquals("Cheese", classes[0].getName());
+        classes = comBlah.getClasses();
+        assertEquals(2, classes.length);
+        assertEquals("Another", classes[0].getName());
+        assertEquals("Thing", classes[1].getName());
+        assertEquals(comBlah, comBlahSubpackage.getParentPackage());
+        assertNull(comBlah.getParentPackage());
+        JavaPackage[] comBlahSubpackages = comBlah.getSubPackages();
+        assertEquals(1, comBlahSubpackages.length);
+        assertEquals(comBlahSubpackage, comBlahSubpackages[0]);
+        JavaPackage[] comBlahSubpackageSubpackages = comBlahSubpackage.getSubPackages();
+        assertEquals(0, comBlahSubpackageSubpackages.length);
+    }
+
     private String createOuter() {
         StringBuffer buffer = new StringBuffer();
         buffer.append("package foo.bar;");
@@ -118,7 +145,7 @@ public class JavaDocBuilderTest extends TestCase {
     }
 
     public void testSourceTree() throws Exception {
-        builder.addSourceTree(new File("tmp/sourcetest"));
+        builder.addSourceTree(new File("target/test-source"));
 
         assertNotNull(builder.getClassByName("com.blah.Thing"));
         assertNotNull(builder.getClassByName("com.blah.Another"));
@@ -126,20 +153,20 @@ public class JavaDocBuilderTest extends TestCase {
     }
 
     public void testRecordFile() throws Exception {
-        builder.addSource(new File("tmp/sourcetest/com/blah/Thing.java"));
+        builder.addSource(new File("target/test-source/com/blah/Thing.java"));
 
         JavaSource[] sources = builder.getSources();
         assertEquals(1, sources.length);
-        assertEquals(new File("tmp/sourcetest/com/blah/Thing.java").toURL(),
+        assertEquals(new File("target/test-source/com/blah/Thing.java").toURL(),
                 sources[0].getURL());
     }
 
     public void testSearcher() throws Exception {
-        builder.addSourceTree(new File("tmp/sourcetest"));
+        builder.addSourceTree(new File("target/test-source"));
 
         List results = builder.search(new Searcher() {
             public boolean eval(JavaClass cls) {
-                return cls.getPackage().equals("com.blah");
+                return cls.getPackage().getName().equals("com.blah");
             }
         });
 
@@ -158,6 +185,20 @@ public class JavaDocBuilderTest extends TestCase {
         writer.close();
     }
 
+    private void deleteDir(String path) {
+        File dir = new File(path);
+        File[] children = dir.listFiles();
+        for (int i = 0; i < children.length; i++) {
+            File file = children[i];
+            if (file.isDirectory()) {
+                deleteDir(file.getAbsolutePath());
+            } else {
+                file.delete();
+            }
+        }
+        dir.delete();
+    }
+    
     public void testDefaultClassLoader() throws Exception {
         String in = ""
                 + "package x;"
@@ -379,7 +420,7 @@ public class JavaDocBuilderTest extends TestCase {
         assertEquals(0, derivedClassesOfArrayList.size());
     }
 
-    public void testSourcePropertyClass() throws IOException, UnsupportedEncodingException {
+    public void testSourcePropertyClass() throws IOException {
         builder.addSource(new File("src/test/com/thoughtworks/qdox/testdata/PropertyClass.java"));
         // Handy way to assert that behaviour for source and binary classes is the same.
         testPropertyClass();
@@ -391,30 +432,75 @@ public class JavaDocBuilderTest extends TestCase {
 
         // test ctor, methods and fields
         JavaMethod[] methods = propertyClass.getMethods();
-        assertEquals(5, methods.length);
+        assertEquals(8, methods.length);
 
         JavaMethod ctor = propertyClass.getMethodBySignature("PropertyClass", null);
+        JavaMethod ctor2 = propertyClass.getMethodBySignature("PropertyClass", new Type[] {propertyClass.asType()});
         JavaMethod getFoo = propertyClass.getMethodBySignature("getFoo", null);
         JavaMethod isBar = propertyClass.getMethodBySignature("isBar", null);
         JavaMethod get = propertyClass.getMethodBySignature("get", null);
         JavaMethod set = propertyClass.getMethodBySignature("set", new Type[]{new Type("int")});
 
+        JavaMethod protectedMethod = propertyClass.getMethodBySignature("protectedMethod", null);
+        JavaMethod privateMethod = propertyClass.getMethodBySignature("privateMethod", null);
+        JavaMethod shouldntBeInherited = propertyClass.getMethodBySignature("getShouldntBeInherited", null);
+
+        assertNotNull(ctor);
+        assertNotNull(ctor2);
+        assertNotNull(getFoo);
+        assertNotNull(isBar);
+        assertNotNull(get);
+        assertNotNull(set);
+        assertNotNull(protectedMethod);
+        assertNotNull(privateMethod);
+        assertNull(shouldntBeInherited);
+
         assertTrue(ctor.isConstructor());
+        assertTrue(ctor2.isConstructor());
         assertFalse(getFoo.isConstructor());
         assertFalse(isBar.isConstructor());
         assertFalse(get.isConstructor());
         assertFalse(set.isConstructor());
+        assertFalse(protectedMethod.isConstructor());
+        assertFalse(privateMethod.isConstructor());
 
         assertTrue(getFoo.isStatic());
         assertFalse(isBar.isStatic());
         assertFalse(get.isStatic());
         assertFalse(set.isStatic());
+        assertFalse(protectedMethod.isStatic());
+        assertFalse(privateMethod.isStatic());
 
         assertTrue(get.isFinal());
-        assertFalse(set.isStatic());
+        assertFalse(set.isFinal());
+
+        assertTrue(ctor2.isProtected());
+        assertTrue(protectedMethod.isProtected());
+        assertTrue(privateMethod.isPrivate());
 
         JavaField[] fields = propertyClass.getFields();
-        assertEquals(1, fields.length);
+        assertEquals(3, fields.length);
+    }
+
+    public void testSourceDefaultCtor() throws Exception {
+        builder.addSource(new File("src/test/com/thoughtworks/qdox/testdata/DefaultCtor.java"));
+        JavaClass javaClass = builder.getClassByName("com.thoughtworks.qdox.testdata.DefaultCtor");
+
+        JavaMethod ctor = javaClass.getMethodBySignature("DefaultCtor", null);
+
+        // Differs from binary as currently no way to identify default
+        // constructor in binary class.
+        assertNull(ctor);
+    }
+
+    public void testBinaryDefaultCtor() {
+        JavaClass javaClass = builder.getClassByName("com.thoughtworks.qdox.testdata.DefaultCtor");
+
+        JavaMethod ctor = javaClass.getMethodBySignature("DefaultCtor", null);
+
+        // Differs from source as currently no way to identify default 
+        // constructor in binary class.
+        assertNotNull(ctor);
     }
 
     public void testSerializable() throws Exception {
@@ -428,8 +514,8 @@ public class JavaDocBuilderTest extends TestCase {
     }
 
     public void testSaveAndRestore() throws Exception {
-        File file = new File("tmp/sourcetest/cache.obj");
-        builder.addSourceTree(new File("tmp/sourcetest"));
+        File file = new File("target/test-source/cache.obj");
+        builder.addSourceTree(new File("target/test-source"));
         builder.save(file);
 
         JavaDocBuilder newBuilder = JavaDocBuilder.load(file);
@@ -737,7 +823,7 @@ public class JavaDocBuilderTest extends TestCase {
         assertEquals(1, results.size());
     } 
     
-    public void FIXME_testJiraQdox63() {
+    public void testJiraQdox63() {
         builder.addSource(
             new StringReader(
                 "package p1;\n" +
@@ -761,10 +847,10 @@ public class JavaDocBuilderTest extends TestCase {
         JavaField innerField = 
             builder.getClassByName("p2.B").getFieldByName("innerField");
         assertEquals(innerClass.asType(), innerField.getType());
-        assertEquals("p1.A$Inner", innerField.getType().getValue());
+        assertEquals("p1.A$Inner", innerField.getType().getFullQualifiedName());
     }
 
-    public void FIXME_testJiraQdox71() {
+    public void testJiraQdox71() {
         String sourceCode = ""
                 + "package foo;"
                 + "public class C {"
@@ -836,6 +922,27 @@ public class JavaDocBuilderTest extends TestCase {
         assertEquals(expected.trim(), javaMethod.getSourceCode().trim());
     }
 
+    public void testMethodBodyWithPrecedingStaticBlock() {
+        JavaDocBuilder builder = new JavaDocBuilder();
+        String sourceCode = "" +
+                "public class X {\n" +
+                "  static {\n" +
+                "    System.out.println(\"static\");\n" +
+                "  }\n" +
+                "  public void doStuff() {\n" +
+                "    System.out.println(\"hi\"); // comment\n" +
+                "    Foo<X> x = new Cheese().get()[4]; /*x*/\n" +
+                "  } // not this \n" +
+                "}";
+        JavaSource javaSource = builder.addSource(new StringReader(sourceCode));
+        JavaClass javaClass = javaSource.getClasses()[0];
+        JavaMethod javaMethod = javaClass.getMethods()[0];
+        String expected = "" +
+                "    System.out.println(\"hi\"); // comment\n" +
+                "    Foo<X> x = new Cheese().get()[4]; /*x*/";
+        assertEquals(expected.trim(), javaMethod.getSourceCode().trim());
+    }
+
     public void testFieldDefinition() {
         JavaDocBuilder builder = new JavaDocBuilder();
         String sourceCode = "" +
@@ -852,4 +959,144 @@ public class JavaDocBuilderTest extends TestCase {
         assertEquals(expected.trim(), javaField.getInitializationExpression().trim());
     }
 
+    public void testNewlessArrays() {
+        String source = "" +
+                "public class Thing {\n" +
+                " long[] bad = {1,2,3};\n" +  // as opposed to bad = new long[] {1,2,3}.
+                "}";
+        JavaDocBuilder builder = new JavaDocBuilder();
+        JavaSource javaSource = builder.addSource(new StringReader(source));
+
+        JavaField field = javaSource.getClasses()[0].getFieldByName("bad");
+        assertEquals("{1,2,3}", field.getInitializationExpression().trim());
+    }
+
+    public void testDefaultsToThrowingExceptionWhenNotParseable() throws Exception {
+        createFile("target/test-source/com/blah/Bad.java", "com.blah", "@%! BAD {}}}}");
+
+        JavaDocBuilder builder = new JavaDocBuilder();
+        try {
+            builder.addSourceTree(new File("target/test-source"));
+            fail("Expected exception");
+        } catch (ParseException expectedException) {
+            // Good!
+        }
+    }
+
+    public void testContinuesProcessingAfterBadFileIfCustomHandlerPermits() throws Exception {
+        createFile("target/test-source/com/blah/Bad.java", "com.blah", "@%! BAD {}}}}");
+
+        Mock mockErrorHandler = mock(JavaDocBuilder.ErrorHandler.class);
+        // Expectation
+        mockErrorHandler.expects(once())
+                .method("handle")
+                .with(isA(ParseException.class));
+
+        JavaDocBuilder builder = new JavaDocBuilder();
+        builder.setErrorHandler((JavaDocBuilder.ErrorHandler) mockErrorHandler.proxy());
+        builder.addSourceTree(new File("target/test-source"));
+
+        assertNotNull(builder.getClassByName("com.blah.Thing"));
+    }
+
+    public void testBinaryClassFieldModifiers() {
+        JavaDocBuilder builder = new JavaDocBuilder();
+        JavaClass javaClass = builder.getClassByName(PropertyClass.class.getName());
+        assertEquals("Class", PropertyClass.class.getName(), javaClass.getFullyQualifiedName());
+        JavaField javaField = javaClass.getFieldByName("aField");
+        assertNotNull("Field", javaField);
+        Set modifiers = new HashSet(Arrays.asList(javaField.getModifiers()));
+        assertEquals("Modifier count", 2, javaField.getModifiers().length);
+        assertTrue("Static", modifiers.contains("static"));
+        assertTrue("Public", modifiers.contains("public"));
+    }
+
+    public void testJiraQdox117() throws Exception {
+    	JavaDocBuilder builder = new JavaDocBuilder();
+        String sourceCode = "" +
+                "public class foo {\n" +
+                "{ dosomething(); }\n" +
+                "@Test (description=\"test blah blah\")\n" +
+                "public void myTest() {}\n" +
+                "}";
+        JavaSource javaSource = builder.addSource(new StringReader(sourceCode));
+        JavaClass javaClass = javaSource.getClasses()[0];
+        JavaMethod javaMethod = javaClass.getMethods()[0];
+        assertEquals("\"test blah blah\"", javaMethod.getAnnotations()[0].getNamedParameter("description").toString());
+    }
+
+    //for qdox-152
+    public void testExtendedClass() throws Exception {
+    	String source = "import javax.faces.component.UIOutput;" +
+    			"public abstract class AbstractSayHello extends UIOutput {\n" +
+    			"}";
+    	JavaSource javaSource = builder.addSource(new StringReader(source));
+        JavaClass javaClass = javaSource.getClasses()[0];
+        assertEquals(javaClass.getSuperClass().getValue(), "javax.faces.component.UIOutput");
+    }
+    
+    //for QDox-154
+    public void todo_testImplicitJavadocCommentOrder() throws Exception {
+        String source = "" +
+        		"public class Foo {\n" +
+        		"    /**\n" + 
+                "     * A Javadoc sample.\n" + 
+                "     *\n" + 
+                "     * @return The size.\n" + 
+                "     */\n" + 
+                "    public long getSize()\n" + 
+                "    {\n" + 
+                "        return 0;\n" + 
+                "    }\n" + 
+                "\n" + 
+                "    /**\n" + 
+                "     * @return The size.\n" +
+                "     *\n" + 
+                "     * A Javadoc sample.\n" + 
+                "     */\n" + 
+                "    public long getSize2()\n" + 
+                "    {\n" + 
+                "        return 0;\n" + 
+                "    }\n" +
+                "}";
+        JavaSource javaSource = builder.addSource(new StringReader(source));
+        JavaClass javaClass = javaSource.getClasses()[0];
+        JavaMethod method1 = javaClass.getMethods()[0];
+        assertEquals( "A Javadoc sample.", method1.getComment());
+        assertEquals( "The size.", method1.getTagByName( "return" ).getValue());
+        JavaMethod method2 = javaClass.getMethods()[1];
+        assertEquals( "The size.", method2.getTagByName( "return" ).getValue());
+        assertEquals( "A Javadoc sample.", method2.getComment());
+    }
+
+    //for qdox-155
+    public void testCharField() throws Exception {
+    	String source = "public class Foo {\n" +
+    			"public static final char SEPARATOR = ',';" +
+    			"}";
+    	JavaSource javaSource = builder.addSource(new StringReader(source));
+        JavaClass javaClass = javaSource.getClasses()[0];
+        assertEquals(javaClass.getFieldByName( "SEPARATOR" ).getInitializationExpression(), "','");
+    }
+    
+  //for qdox-157
+    public void testCommentBetweenFields() throws Exception {
+    	String source = "public class Foo {\n" +
+    			"public static final String TEST1 = \"test1\";\n" +
+    			"// TODO: blabla\n" +
+    			"public static final String TEST2 = \"test2\";\n" +
+    			"}";
+    	JavaSource javaSource = builder.addSource(new StringReader(source));
+    	JavaClass javaClass = javaSource.getClasses()[0];
+        assertEquals(javaClass.getFieldByName( "TEST2" ).getInitializationExpression(), "\"test2\"");
+    }
+    
+    public void testAnnotationWithComment() throws Exception {
+    	String source = "@OneToMany(cascade = {/* CascadeType.PERSIST */}, fetch = FetchType.LAZY)\n" +
+    			"public class Foo{}"; 
+    	JavaSource javaSource = builder.addSource(new StringReader(source));
+    	JavaClass javaClass = javaSource.getClasses()[0];
+    	assertNotNull(javaClass.getAnnotations()[0].getNamedParameter("cascade"));
+    } 
+    
 }
