@@ -2,6 +2,8 @@ package com.thoughtworks.qdox.model;
 
 import junit.framework.TestCase;
 
+import java.util.HashMap;
+
 public class JavaClassTest extends TestCase {
 
     private JavaClass cls;
@@ -18,46 +20,46 @@ public class JavaClassTest extends TestCase {
         src.addClass(cls);
     }
 
-    public void testToStringSimpleClass() throws Exception {
+    public void testGetCodeBlockSimpleClass() throws Exception {
         cls.setName("MyClass");
         String expected = ""
                 + "class MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringSimpleInterface() throws Exception {
+    public void testGetCodeBlockSimpleInterface() throws Exception {
         cls.setName("MyClass");
         cls.setInterface(true);
         String expected = ""
                 + "interface MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringSimpleEnum() throws Exception {
+    public void testGetCodeBlockSimpleEnum() throws Exception {
         cls.setName("MyEnum");
         cls.setEnum(true);
         String expected = ""
                 + "enum MyEnum {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassExtends() throws Exception {
+    public void testGetCodeBlockClassExtends() throws Exception {
         cls.setName("MyClass");
         cls.setSuperClass(new Type("SuperClass"));
         String expected = ""
                 + "class MyClass extends SuperClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringInterfaceExtends() throws Exception {
+    public void testGetCodeBlockInterfaceExtends() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface"}));
         cls.setInterface(true);
@@ -65,10 +67,10 @@ public class JavaClassTest extends TestCase {
                 + "interface MyClass extends SomeInterface {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringInterfaceExtendsTwo() throws Exception {
+    public void testGetCodeBlockInterfaceExtendsTwo() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface", "AnotherInterface"}));
         cls.setInterface(true);
@@ -76,10 +78,10 @@ public class JavaClassTest extends TestCase {
                 + "interface MyClass extends SomeInterface, AnotherInterface {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringInterfaceExtendsThree() throws Exception {
+    public void testGetCodeBlockInterfaceExtendsThree() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface", "AnotherInterface", "Thingy"}));
         cls.setInterface(true);
@@ -87,30 +89,30 @@ public class JavaClassTest extends TestCase {
                 + "interface MyClass extends SomeInterface, AnotherInterface, Thingy {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassImplements() throws Exception {
+    public void testGetCodeBlockClassImplements() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface"}));
         String expected = ""
                 + "class MyClass implements SomeInterface {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassImplementsTwo() throws Exception {
+    public void testGetCodeBlockClassImplementsTwo() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface", "AnotherInterface", "Xx"}));
         String expected = ""
                 + "class MyClass implements SomeInterface, AnotherInterface, Xx {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassImplementsAndExtends() throws Exception {
+    public void testGetCodeBlockClassImplementsAndExtends() throws Exception {
         cls.setName("MyClass");
         cls.setImplementz(type(new String[]{"SomeInterface", "AnotherInterface", "Xx"}));
         cls.setSuperClass(new Type("SubMarine"));
@@ -118,37 +120,37 @@ public class JavaClassTest extends TestCase {
                 + "class MyClass extends SubMarine implements SomeInterface, AnotherInterface, Xx {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringModifers() throws Exception {
+    public void testGetCodeBlockModifers() throws Exception {
         cls.setName("MyClass");
         cls.setModifiers(new String[]{"public", "final"});
         String expected = ""
                 + "public final class MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringModifersProtectionAlwaysFirst() throws Exception {
+    public void testGetCodeBlockModifersProtectionAlwaysFirst() throws Exception {
         cls.setName("MyClass");
         cls.setModifiers(new String[]{"final", "public"});
         String expected = ""
                 + "public final class MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
 
         cls.setModifiers(new String[]{"abstract", "protected"});
         expected = ""
                 + "protected abstract class MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithOneMethod() throws Exception {
+    public void testGetCodeBlockClassWithOneMethod() throws Exception {
         cls.setName("MyClass");
         JavaMethod mth = new JavaMethod();
         mth.setName("doStuff");
@@ -160,10 +162,10 @@ public class JavaClassTest extends TestCase {
                 + "\tvoid doStuff();\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithThreeMethods() throws Exception {
+    public void testGetCodeBlockClassWithThreeMethods() throws Exception {
         cls.setName("MyClass");
 
         {
@@ -197,10 +199,10 @@ public class JavaClassTest extends TestCase {
                 + "\tvoid eat();\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithTwoFields() throws Exception {
+    public void testGetCodeBlockClassWithTwoFields() throws Exception {
         cls.setName("MyClass");
 
         {
@@ -226,10 +228,10 @@ public class JavaClassTest extends TestCase {
                 + "\tpublic String thing;\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithInnerClass() throws Exception {
+    public void testGetCodeBlockClassWithInnerClass() throws Exception {
         cls.setName("Outer");
         JavaClass innerClass = new JavaClass();
         innerClass.setName("Inner");
@@ -243,10 +245,10 @@ public class JavaClassTest extends TestCase {
                 + "\t}\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithInnerEnum() throws Exception {
+    public void testGetCodeBlockClassWithInnerEnum() throws Exception {
         cls.setName("Outer");
         JavaClass innerEnum = new JavaClass();
         innerEnum.setEnum(true);
@@ -261,10 +263,10 @@ public class JavaClassTest extends TestCase {
                 + "\t}\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringEnumWithInnerClass() throws Exception {
+    public void testGetCodeBlockEnumWithInnerClass() throws Exception {
         cls.setName("Outer");
         cls.setEnum(true);
         JavaClass innerClass = new JavaClass();
@@ -279,11 +281,11 @@ public class JavaClassTest extends TestCase {
                 + "\t}\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
 
-    public void testToStringClassWithComment() throws Exception {
+    public void testGetCodeBlockClassWithComment() throws Exception {
         cls.setName("MyClass");
         cls.setComment("Hello World");
 
@@ -294,10 +296,10 @@ public class JavaClassTest extends TestCase {
                 + "class MyClass {\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
-    public void testToStringClassWithIndentedCommentsForFieldAndMethod() throws Exception {
+    public void testGetCodeBlockClassWithIndentedCommentsForFieldAndMethod() throws Exception {
         cls.setName("MyClass");
         cls.setComment("Hello World");
 
@@ -330,7 +332,7 @@ public class JavaClassTest extends TestCase {
                 + "\tString thingy();\n"
                 + "\n"
                 + "}\n";
-        assertEquals(expected, cls.toString());
+        assertEquals(expected, cls.getCodeBlock());
     }
 
     public void testIsPublic() {
@@ -342,12 +344,12 @@ public class JavaClassTest extends TestCase {
     }
 
     public void testQualifiedType() throws Exception {
-        src.setPackage("com.thoughtworks.qdox");
+        src.setPackage(new JavaPackage("com.thoughtworks.qdox", new HashMap()));
 
         cls.setName("MyClass");
 
         assertEquals("MyClass", cls.getName());
-        assertEquals("com.thoughtworks.qdox", cls.getPackage());
+        assertEquals("com.thoughtworks.qdox", cls.getPackage().getName());
         assertEquals("com.thoughtworks.qdox.MyClass",
                 cls.getFullyQualifiedName());
         assertTrue(cls.asType().isResolved());
@@ -355,13 +357,13 @@ public class JavaClassTest extends TestCase {
     }
 
     public void testGetClassNamePrefix() {
-        src.setPackage("foo.bar");
+        src.setPackage(new JavaPackage("foo.bar", new HashMap()));
         cls.setName("Stanley");
         assertEquals("foo.bar.Stanley$", cls.getClassNamePrefix());
     }
     
     public void testInnerClass() throws Exception {
-        src.setPackage("foo.bar");
+        src.setPackage(new JavaPackage("foo.bar", new HashMap()));
 
         JavaClass outer = new JavaClass();
         outer.setName("Outer");
@@ -372,7 +374,7 @@ public class JavaClassTest extends TestCase {
         outer.addClass(inner);
 
         assertEquals("Inner", inner.getName());
-        assertEquals("foo.bar", inner.getPackage());
+        assertEquals("foo.bar", inner.getPackage().getName());
         assertEquals("foo.bar.Outer$Inner",
                 inner.getFullyQualifiedName());
     }
@@ -460,7 +462,7 @@ public class JavaClassTest extends TestCase {
     }
     
     public void testResolveTypeInnerClass() throws Exception {
-        src.setPackage("p");
+        src.setPackage(new JavaPackage("p", new HashMap()));
         cls.setName("X");
         JavaClass innerClass = new JavaClass();
         innerClass.setName("DogFood");
@@ -492,6 +494,101 @@ public class JavaClassTest extends TestCase {
         assertEquals(getFooMethod, fooProp.getAccessor());
         assertEquals(setFooMethod, fooProp.getMutator());
     }
+    
+    public void testToStringClass() {
+    	cls.setName("com.MyClass");
+    	assertEquals("class com.MyClass", cls.toString());
+    }
+    
+    public void testInnerClassToString() throws Exception {
+    	JavaPackage jPackage = new JavaPackage("com.thoughtworks.qdox.model");
+    	JavaClass jOuterClass = new JavaClass("OuterClass");
+    	jPackage.addClass(jOuterClass);
+    	JavaClass jInnerClass = new JavaClass("InnerClass");
+    	jOuterClass.addClass(jInnerClass);
+    	assertEquals("class com.thoughtworks.qdox.model.OuterClass$InnerClass", jInnerClass.toString());
+    }
+    
+    public void testInnerClassType() {
+        JavaPackage jPackage = new JavaPackage("com.thoughtworks.qdox.model");
+        JavaClass jOuterClass = new JavaClass("OuterClass");
+        jPackage.addClass(jOuterClass);
+        JavaClass jInnerClass = new JavaClass("InnerClass");
+        jOuterClass.addClass(jInnerClass);
+        assertEquals("com.thoughtworks.qdox.model.OuterClass.InnerClass", jInnerClass.asType().getValue());
+    }
+    
+    public void testInnerInterfaceToString() {
+    	JavaPackage jPackage = new JavaPackage("com.thoughtworks.qdox.model");
+    	JavaClass jOuterClass = new JavaClass("OuterClass");
+    	jPackage.addClass(jOuterClass);
+    	JavaClass jInnerInterface = new JavaClass("InnerInterface");
+    	jInnerInterface.setInterface(true);
+    	jOuterClass.addClass(jInnerInterface);
+    	assertEquals("interface com.thoughtworks.qdox.model.OuterClass$InnerInterface", jInnerInterface.toString());
+    }
+    
+    public void testToStringInterface() {
+    	cls.setName("com.MyClass");
+    	cls.setInterface(true);
+    	assertEquals("interface com.MyClass", cls.toString());
+    }
+    
+    public void testToStringVoid() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(Type.VOID, "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("void", javaMethod.getReturns().toString());
+    }
+
+    public void testToStringBoolean() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("boolean"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("boolean", javaMethod.getReturns().toString());
+    }
+    
+    public void testToStringInt() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("int"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("int", javaMethod.getReturns().toString());
+    }
+
+    public void testToStringLong() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("long"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("long", javaMethod.getReturns().toString());
+    }
+
+    public void testToStringFloat() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("float"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("float", javaMethod.getReturns().toString());
+    }
+
+    public void testToStringDouble() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("double"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("double", javaMethod.getReturns().toString());
+    }
+    
+    public void testToStringChar() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("char"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("char", javaMethod.getReturns().toString());
+    }
+
+    public void testToStringByte() {
+    	cls.setName("com.MyClass");
+    	cls.addMethod(new JavaMethod(new Type("byte"), "doSomething"));
+    	JavaMethod javaMethod = cls.getMethods()[0];
+    	assertEquals("byte", javaMethod.getReturns().toString());
+    }
 
     /**
      * @codehaus.jira QDOX-59
@@ -515,4 +612,6 @@ public class JavaClassTest extends TestCase {
         }
         return result;
     }
+    
+    
 }
