@@ -528,8 +528,7 @@ public class LexerTest extends TestCase {
         Lexer lexer = new JFlexLexer(new StringReader(in));
 
         assertLex(Parser.PUBLIC, lexer);
-        assertLex(Parser.AT, lexer);
-        assertLex(Parser.INTERFACE, lexer);
+        assertLex(Parser.ANNOINTERFACE, lexer);
         assertLex(Parser.IDENTIFIER, "Copyright", lexer);
         assertLex(Parser.BRACEOPEN, lexer);
         assertLex(Parser.IDENTIFIER, "int", lexer);
@@ -548,21 +547,28 @@ public class LexerTest extends TestCase {
 
     public void testAnnotationTokens() throws Exception {
         String in = ""
-            + "@Copyright (year = 2004, month = \"Jan\")\n" 
+            + "@Copyright (year = 2004, month = \"Jan\")\n"
             + "@Note(\"Just ignore me\")\n" 
             + "public class LexerTest extends TestCase {}\n";
         Lexer lexer = new JFlexLexer(new StringReader(in));
 
-        assertLex(Parser.AT, lexer);
+        assertLex(Parser.AT, "@", lexer);
         assertLex(Parser.IDENTIFIER, "Copyright", lexer);
         assertLex(Parser.PARENOPEN, lexer);
         assertLex(Parser.IDENTIFIER, "year", lexer);
+        assertLex(Parser.EQUALS, lexer);
+        assertLex(Parser.INTEGER_LITERAL, "2004", lexer);
         assertLex(Parser.COMMA, lexer);
         assertLex(Parser.IDENTIFIER, "month", lexer);
+        assertLex(Parser.EQUALS, lexer);
+        assertLex(Parser.STRING_LITERAL, "\"", lexer);
+        // TODO the above should be ...
+        // assertLex(Parser.STRING_LITERAL, "Jan", lexer);
         assertLex(Parser.PARENCLOSE, lexer);
-        assertLex(Parser.AT, lexer);
+        assertLex(Parser.AT, "@", lexer);
         assertLex(Parser.IDENTIFIER, "Note", lexer);
         assertLex(Parser.PARENOPEN, lexer);
+        assertLex(Parser.STRING_LITERAL, lexer);
         assertLex(Parser.PARENCLOSE, lexer);
         assertLex(Parser.PUBLIC, lexer);
         assertLex(Parser.CLASS, lexer);
@@ -574,6 +580,29 @@ public class LexerTest extends TestCase {
         assertLex(0, lexer);
     }
 
+    public void testQDOX134_MoreAnnotationTokens() throws Exception {
+        String in = ""
+            + "@myTag name=TestClass attrs=Something1,Something2,Something3\n"
+            + "public class LexerTest extends TestCase {}\n";
+        Lexer lexer = new JFlexLexer(new StringReader(in));
+
+        assertLex(Parser.AT, "@", lexer);
+        assertLex(Parser.IDENTIFIER, "myTag", lexer);
+        assertLex(Parser.IDENTIFIER, "name", lexer);
+        assertLex(Parser.COMMA, lexer);
+        assertLex(Parser.IDENTIFIER, "Something2", lexer);
+        assertLex(Parser.COMMA, lexer);
+        assertLex(Parser.IDENTIFIER, "Something3", lexer);
+        assertLex(Parser.PUBLIC, lexer);
+        assertLex(Parser.CLASS, lexer);
+        assertLex(Parser.IDENTIFIER, "LexerTest", lexer);
+        assertLex(Parser.EXTENDS, lexer);
+        assertLex(Parser.IDENTIFIER, "TestCase", lexer);
+        assertLex(Parser.BRACEOPEN, lexer);
+        assertLex(Parser.BRACECLOSE, lexer);
+        assertLex(0, lexer);
+    }
+
     public void testEnumConstructor() throws Exception {
         String in = "enum Foo { a(\"hello\"); int someField; }";
         Lexer lexer = new JFlexLexer(new StringReader(in));
@@ -621,4 +650,94 @@ public class LexerTest extends TestCase {
         assertEquals(expectedText, lexer.text());
     }
 
+    // for QDOX-140
+    public void testNonAsciiMethodNameDoesNotCrashLexerButChewsUpUnicodeEscapedSequencesBadly() throws Exception {
+        String in = ""
+                + "interface X { "
+                + "   void paramWithNonAsciis\\u00E4\\u00F6\\u00FC\\u00DF();"
+                + "} ";
+
+        Lexer lexer = new JFlexLexer(new StringReader(in));
+        assertLex(Parser.INTERFACE, lexer);
+        assertLex(Parser.IDENTIFIER, lexer);
+        assertLex(Parser.BRACEOPEN, lexer);
+        assertLex(Parser.IDENTIFIER, "void", lexer);
+        assertLex(Parser.IDENTIFIER, "paramWithNonAsciis\\u00E4\\u00F6\\u00FC\\u00DF", lexer);
+        assertLex(Parser.PARENOPEN, lexer);
+        assertLex(Parser.PARENCLOSE, lexer);
+        assertLex(Parser.SEMI, lexer);
+        assertLex(Parser.BRACECLOSE, lexer);
+        assertLex(0, lexer);
+    }
+    
+    //for QDOX-158
+    public void testAnnotationWithMultipleParameters() throws Exception {
+    	String in = 
+    			"@MyFunction.MyInterface( prefix1 = \"abc\", prefix2 = \"abc\" )";
+    	Lexer lexer = new JFlexLexer(new StringReader(in));
+    	assertLex(Parser.AT, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.DOT, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENOPEN, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.EQUALS, lexer);
+    	assertLex(Parser.STRING_LITERAL, lexer);
+    	assertLex(Parser.COMMA, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.EQUALS, lexer);
+    	assertLex(Parser.STRING_LITERAL, lexer);
+    	assertLex(Parser.PARENCLOSE, lexer);
+    }
+
+    public void testSimpleAnnotation() throws Exception {
+    	String in = "@Override\n public boolean isReadOnly(final ELContext context)";
+    	Lexer lexer = new JFlexLexer(new StringReader(in));
+    	assertLex(Parser.AT, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PUBLIC, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENOPEN, lexer);
+    	assertLex(Parser.FINAL, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENCLOSE, lexer);
+    }
+    
+    public void testMultipleRowAnnotation() throws Exception {
+    	String in = "@JSFComponent\n  (name = \"h:inputHidden\")";
+    	Lexer lexer = new JFlexLexer(new StringReader(in));
+    	assertLex(Parser.AT, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENOPEN, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.EQUALS, lexer);
+    	assertLex(Parser.STRING_LITERAL, lexer);
+    	assertLex(Parser.PARENCLOSE, lexer);
+    }
+    
+    public void testEnumWithAnnotations() throws Exception {
+    	String in = "class Foo {\n" +
+		"public enum BasicType {\n" +
+		"@XmlEnumValue(\"text\")\n" +
+		"VALUE(\"value\"); }\n" +
+		"}";  
+    	Lexer lexer = new JFlexLexer(new StringReader(in));
+    	assertLex(Parser.CLASS, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.BRACEOPEN, lexer);
+    	assertLex(Parser.PUBLIC, lexer);
+    	assertLex(Parser.ENUM, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.BRACEOPEN, lexer);
+    	assertLex(Parser.AT, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENOPEN, lexer);
+    	assertLex(Parser.STRING_LITERAL, lexer);
+    	assertLex(Parser.PARENCLOSE, lexer);
+    	assertLex(Parser.IDENTIFIER, lexer);
+    	assertLex(Parser.PARENBLOCK, lexer);
+    	assertLex(Parser.SEMI, lexer);
+    }
 }
